@@ -24,14 +24,31 @@ def main():
 
     api = solaredgeoptimizers(siteId, username, password)
 
-    if api.check_login():
+    test = api.requestListOfAllPanels()
 
-        alles = api.requestAllData()
+    try:
+        print(len(test.inverters))
+    except Exception as ex:
+        print("error")
+
+##
+##    i = 1
+##    for inverter in test.inverters:
+##        print("Adding all optimizers from inverter: %s", i)
+##        for string in inverter.strings:
+##            for optimizer in string.optimizers:
+##                print("Added optimizer for panel_id to Home Assistant",optimizer.displayName)
+##
+##        i=i+1
+
+    #if api.check_login():
+
+#        alles = api.requestAllData()
 
         #wtf = api.getLifeTimeEnergy()
         #print(wtf)
-        for paneel in alles:
-            print(paneel.lifetime_energy)
+        #for paneel in alles:
+            #print(paneel.lifetime_energy)
 
         #panelen = api.requestListOfAllPanels().ReturnAllPanelsIds()
 
